/*-
 * Copyright (c) 2001-2005 Christian S.J. Peron
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ifndef lint
static const char rcsid[] =
        "@(#) $Header: /usr/cvs/ipex/dev_mem_ops.c,v 1.5 2005/02/25 08:46:11 modulus Exp $";
#endif

#include "ipex_includes.h"

#ifndef linux

static struct op_lookup kops[] = {
   { KERN_PROC_PID,          "pid"  },
   { KERN_PROC_UID,          "uid" },
   { KERN_PROC_RUID,         "ruid"  },
   { 0,                          0  },
};


int 
get_dev_mem_op(key)
   char *key;
{
    struct op_lookup *p;

   for(p = kops; p->op; ++p)
      if (strncmp(p->key, key, 4) == 0)
         return(p->op);

  (void) fprintf(stderr,"fatal: invalid -P switch\n");

  exit(0);
}

struct core *
proc_args(char *arg)
{
	char *op;
	struct core *uap;

	if (strchr(arg, '=') == NULL) {
		printf("fatal: invalid -P switch\n");
		return(NULL);
	}

	op = strsep(&arg, "=");

	uap = malloc(sizeof(struct core));

	if ((uap->op = get_dev_mem_op(op)) == 0) {
		printf("fatal: NOOP: no such operation.\n");
                exit (-1);
	}
        if (strisdigit(arg))
        {
                printf ("fatal: -P argument needs to be all digits.\n");
                exit (-1);
        }

	uap->arg = atoi(arg);

	return(uap);
}

#endif /* linux */
